/*
 * This file is a part of KleanSweep.
 *
 * Copyright (C) 2005 Pawel Stolowski <pawel.stolowski@wp.pl>
 *
 * KleanSweep is free software; you can redestribute it and/or modify it
 * under terms of GNU General Public License by Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY. See GPL for more details.
 */

#include "sweepscanner.h"
#include "sweepfileinfo.h"
#include <kprocess.h>
#include <stdlib.h>

#define MAX_LINE_LEN 1024

SweepScanner::SweepScanner(): QObject(), lineptr(0), running(false), ftypes_changed(true)
{
        line = new char [MAX_LINE_LEN];
	for (int i=0; i<NONE; i++)
		ftypes[i] = false;
	files.setAutoDelete(true);
        
	proc = new KProcess(this);
}

SweepScanner::~SweepScanner()
{
	stop();
        delete line;
}

void SweepScanner::enableFileType(FileType f, bool enable)
{
	if (ftypes[f] != enable)
	{
		ftypes[f] = enable;
		ftypes_changed = true;
	}
}

bool SweepScanner::isEnabled(FileType f)
{
	return ftypes[f];
}

void SweepScanner::clear()
{
	files.clear();
	dupmap.clear();
}

bool SweepScanner::start(const QString &path)
{
	if (proc->isRunning())
		return false;

	clear();
	ftypes_changed = false;
	
        connect(proc, SIGNAL(receivedStdout(KProcess *, char *, int)), this, SLOT(onProcessData(KProcess *, char *, int)));

        *proc << "kleansweep-helper";
	if (isEnabled(EMPTY_FILE))      *proc << "-z";
	if (isEnabled(EMPTY_DIR))       *proc << "-y";
	if (isEnabled(BROKEN_SYMLINK))  *proc << "-s";
	if (isEnabled(DUPLICATED_FILE)) *proc << "-d";
	if (isEnabled(RPM_ORPHAN))      *proc << "-R";
	if (isEnabled(DEB_ORPHAN))      *proc << "-D";
	if (isEnabled(EMERGE_ORPHAN))   *proc << "-E";
	if (isEnabled(DEAD_MENUENTRY))  *proc << "-m";
	if (isEnabled(DEAD_THUMBNAIL))  *proc << "-t";
	if (isEnabled(UNUSED_LOCALE))   *proc << "-l";
	*proc << path;
        proc->setPriority(10);
	connect(proc, SIGNAL(processExited(KProcess *)), this, SLOT(helperExited(KProcess *)));
        if (!proc->start(KProcess::NotifyOnExit, KProcess::Stdout))
	{
		delete proc;
		return false;
	}
	last_path = path;
	return true;
}

bool SweepScanner::isRunning()
{
	return proc->isRunning();
}

bool SweepScanner::stop()
{
	if (isRunning())
		proc->kill(SIGINT);
	return true;
}

void SweepScanner::onProcessData(KProcess *proc, char *buf, int len)
{
        for (int i=0; i<len; i++)
        {
                if (buf[i] == '\n')
                {
                        line[lineptr] = 0;
                        lineptr = 0;
                        extractFileInfo();
                }
                else
                {
                        line[lineptr++] = buf[i];
                        if (lineptr == MAX_LINE_LEN-1)
                        {
                                ;
                        }
                }
        }
}

void SweepScanner::extractFileInfo()
{
        char tags[32]; //should be enough...
	char target[1024];
	
	int pos = 0;
	int i = 0;

	//
	// find target start (skip tags)
        while (line[pos] && line[pos]!=':')
                ++pos;
	
	++pos; //skip ':'

	//
	// parse target
	while (line[pos])
	{
		if (line[pos] == ':')
		{
			target[i++] = 0;
			++pos;
			break;
		}
		if (line[pos] == '\\') //backslash found - take next character 'as is'
			target[i++] = line[++pos];
		else
			target[i++] = line[pos];
		++pos;
	}
	
	SweepFileInfo *f = new SweepFileInfo(line + pos);
	files.append(f);
        
	/*if (tags[0] == 'a')
		score = 10;
	else
		score = tags[0] - '0';
	f->setScore(score);*/

        for (int i=0; line[i]!=':'; i++)
        { 
		FileType ft;
		switch (line[i])
		{
			case 'y': ft = EMPTY_DIR; break;
			case 'z': ft = EMPTY_FILE; break;
			case 'l': ft = UNUSED_LOCALE; break;
			case 'b': ft = BACKUP_FILE; break;
			case 'm': ft = DEAD_MENUENTRY; break;
			case 'd': ft = DUPLICATED_FILE; break;
			case 's': ft = BROKEN_SYMLINK; break;
			case 't': ft = DEAD_THUMBNAIL; break;
			case 'R': ft = RPM_ORPHAN; break;
			case 'D': ft = DEB_ORPHAN; break;
			case 'E': ft = EMERGE_ORPHAN; break;
			default: ft = NONE; break;
		}
		if (ft == DEAD_MENUENTRY || ft == DEAD_THUMBNAIL || ft == BROKEN_SYMLINK)
		{
			f->setTarget(target);
		}
		else if (ft == DUPLICATED_FILE)
		{
			const long id = atol(tags + (++i));
			if (dupmap.contains(id))
			{
				emit newFileEntry(ft, f, dupmap[id]);
			}
			else
			{
				dupmap.insert(id, f);
				emit newFileEntry(ft, f);
			}
			break;
		}
		if (ft != NONE && ftypes[ft])
			emit newFileEntry(ft, f);
        }
}

void SweepScanner::helperExited(KProcess *proc)
{
	emit finished(true);
	proc->deleteLater();
	running = false;
	dupmap.clear();
}

QPtrList<SweepFileInfo> SweepScanner::getFilesForRemoval()
{
	QPtrList<SweepFileInfo> remfiles;
	remfiles.setAutoDelete(false);
	for (SweepFileInfo *finfo = files.first(); finfo; finfo = files.next())
		if (finfo->isMarkedForRemoval())
			remfiles.append(finfo);
	return remfiles;
}

bool SweepScanner::settingsChanged() const
{
	return ftypes_changed;
}

QString SweepScanner::lastScanPath() const
{
	return last_path;
}

#include "sweepscanner.moc"
